@@ -1,5 +1,46 @@
 #include "RTMB.h"
 
+inline Rcpp::ComplexVector unwrap(ADrep x) {
+  return Rcpp::ComplexVector(x);
+}
+inline Rcpp::ComplexMatrix unwrap_matrix(ADrep x) {
+  return Rcpp::ComplexMatrix(x);
+}
+void ADrep::setclass () {
+  Base::operator=( Rf_asS4(*this, TRUE, FALSE) );
+  (*this).attr("class") = "advector";
+}
+// Default CTOR
+ADrep::ADrep () {}
+// Input
+ADrep::ADrep (Rcpp::RObject x) : Rcpp::RObject(x) {
+  CHECK_INPUT(*this);
+}
+// Output
+ADrep::ADrep (size_t n) {
+  Base::operator=( Rcpp::ComplexVector(n) );
+  this -> setclass();
+}
+ADrep::ADrep (const ad* begin, const ad* end) {
+  Base::operator=( Rcpp::ComplexVector((Rcomplex*) begin, (Rcomplex*) end) );
+  this -> setclass();
+}
+ADrep::ADrep (size_t n, size_t m) {
+  Base::operator=( Rcpp::ComplexMatrix(n, m) );
+  this -> setclass();
+}
+ad* ADrep::adptr() {
+  static_assert(sizeof(ad) == sizeof(Rcomplex),
+                "ad size must match Rcomplex");
+  Rcpp::ComplexVector x = unwrap(*this);
+  ad* px = (x.size() > 0 ? (ad*)(x.begin()) : NULL);
+  return px;
+}
+size_t ADrep::size() { return unwrap(*this).size(); }
+size_t ADrep::nrow() { return unwrap_matrix(*this).nrow(); }
+size_t ADrep::ncol() { return unwrap_matrix(*this).ncol(); }
+
+
 Rcomplex ad2cplx(const ad &x) {
   static_assert(sizeof(ad) == sizeof(Rcomplex),
                 "ad size must match Rcomplex");
@@ -12,11 +53,8 @@ ad cplx2ad(const Rcomplex &x) {
   ad* px = (ad*)(&x);
   return *px;
 }
-ad* adptr(const Rcpp::ComplexVector &x) {
-  static_assert(sizeof(ad) == sizeof(Rcomplex),
-                "ad size must match Rcomplex");
-  ad* px = (x.size() > 0 ? (ad*)(&(x[0])) : NULL);
-  return px;
+ad* adptr(ADrep x) {
+  return x.adptr();
 }
 bool is_advector (SEXP x) {
   return Rf_inherits(x, "advector");
@@ -37,9 +75,11 @@ bool valid(const ad &x) {
     !x.ontape() || x.in_context_stack(x.data.glob);
 }
 // [[Rcpp::export]]
-bool valid(Rcpp::ComplexVector x) {
-  for (int i=0; i<x.size(); i++)
-    if (!valid(cplx2ad(x[i]))) return false;
+bool valid(ADrep x) {
+  ad* X = x.adptr();
+  size_t n = x.size();
+  for (size_t i=0; i<n; i++)
+    if (!valid(X[i])) return false;
   return true;
 }
 // [[Rcpp::export]]
@@ -55,23 +95,20 @@ Rcpp::ComplexVector& as_advector(Rcpp::ComplexVector &x) {
 }
 
 // ============================== Dense matrices
-ConstMapMatrix MatrixInput(const Rcpp::ComplexMatrix &x) {
-  return ConstMapMatrix ((ad*) x.begin(), x.nrow(), x.ncol());
+ConstMapMatrix MatrixInput(ADrep x) {
+  return ConstMapMatrix (x.adptr(), x.nrow(), x.ncol());
 }
-Rcpp::ComplexMatrix MatrixOutput(const matrix<ad> &X) {
-  Rcpp::ComplexMatrix z(X.rows(), X.cols());
-  MapMatrix Z((ad*) z.begin(), z.nrow(), z.ncol());
+ADrep MatrixOutput(const matrix<ad> &X) {
+  ADrep z(X.rows(), X.cols());
+  MapMatrix Z(z.adptr(), z.nrow(), z.ncol());
   Z = X;
-  // FIXME: z = as_advector(z);
-  z = Rf_asS4(z, TRUE, FALSE); // Was: SET_S4_OBJECT(z);
-  z.attr("class") = "advector";
   return z;
 }
 
 // ============================== Sparse matrices
-Eigen::SparseMatrix<ad> SparseInput(Rcpp::S4 S) {
-  Rcpp::ComplexVector x(S.slot("x"));
-  CHECK_INPUT(x);
+Eigen::SparseMatrix<ad> SparseInput(Rcpp::RObject S_) {
+  Rcpp::S4 S(S_);
+  ADrep x(Rcpp::RObject(S.slot("x")));
   Rcpp::IntegerVector i = S.slot("i");
   Rcpp::IntegerVector p = S.slot("p");
   Rcpp::IntegerVector Dim = S.slot("Dim");
@@ -81,30 +118,28 @@ Eigen::SparseMatrix<ad> SparseInput(Rcpp::S4 S) {
                                                 i.size(), // nonZeros()
                                                 p.begin(), // outerIndexPtr()
                                                 i.begin(), // innerIndexPtr()
-                                                (ad*) x.begin(), // data()
+                                                x.adptr(), // data()
                                                 NULL); // innerNonZeroPtr();
 }
 
-Rcpp::S4 SparseOutput (const Eigen::SparseMatrix<ad> &S) {
+Rcpp::RObject SparseOutput (const Eigen::SparseMatrix<ad> &S) {
   size_t nnz  = S.nonZeros();
   Rcpp::IntegerVector Dim(2);
   Dim[0] = S.rows();
   Dim[1] = S.cols();
   Rcpp::IntegerVector i(S.innerIndexPtr(), S.innerIndexPtr() + nnz);
   Rcpp::IntegerVector p(S.outerIndexPtr(), S.outerIndexPtr() + Dim[1] + 1);
-  Rcpp::ComplexVector x( (Rcomplex*) (S.valuePtr()),
-                         (Rcomplex*) (S.valuePtr() + nnz));
+  ADrep x( S.valuePtr(), S.valuePtr() + nnz);
   Rcpp::S4 ans("adsparse");
-  ans.slot("x") = as_advector(x);
+  ans.slot("x") = x;
   ans.slot("i") = i;
   ans.attr("p") = p;
   ans.attr("Dim") = Dim;
-  return ans;
+  return Rcpp::RObject(ans);
 }
 
 ad ScalarInput(SEXP x_) {
-  Rcpp::ComplexVector x(x_);
-  CHECK_INPUT(x);
-  return cplx2ad(x[0]);
+  ADrep x(x_);
+  return adptr(x)[0];
 }
 
