@@ -1,3 +1,6 @@
+// TMB already included => skip!
+#ifndef TMB_OBJECTIVE_PTR
+
 #include "config.h"
 #ifndef TMB_H
 #define TMB_H
@@ -34,3 +37,5 @@
 #endif
 #include <TMB.hpp>
 #endif
+
+#endif
